		TITLE			"AH4 Tuner Control Program Rev. 5"
		SUBTITLE		"Module Header"
;**********************************************************************
;   This file is a basic code template for assembly code generation   *
;   on the PICmicro PIC16C54B. This file contains the basic code      *
;   building blocks to build upon.                                    *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    ah4.asm                                           *
;    Date:          30 Mar 2002                                       *
;    Release:       5                                                 *
;                                                                     *
;    Author:        Chris Petersen, K9EQ                              *
;    CVS: $log:$
;                                                                     * 
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;    p16C5x.inc                                                       *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;                                                                     *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    History:    
;    30 Mar 2002 Rev. 5   
;       Added untune disables DTS PTT of HF rig                                              
;    10 Mar 2002 Rev. 4
;       Added control line for PSK31 PTT
;       Added control line for ALC output
;    13 Jan 2002 Rev. 3                                               *
;       Error flash is slower                                         *
;       Delay before checking for rising AH-4 Key after tune this     *
;          was required to deal with slow falling edge                *
;       Smooth out blinking transition between phases                 *
;**********************************************************************

		SUBTITLE		"Pseudo Code"
		page
;**********************************************************************
; Psuedo code
;
; Enter 'A' at power-up. Initialize all hardware
; Sit in idle loop waiting for either the UNTUNE or TUNE buttons
; Flash the LED is error status is set
;
; A1.  Enter at Power On Reset (POR)
;
; A2.  Initialize:
;      Initialize I/O ports for input/output
;      Turn LED off
;      Disable Tx
;      Disable Start
;      Clear error status
;
; A2a. Check for Demo mode
;
; A3.  Check for tune key
;      If detected goto state C1
;
; A4.  Check for untune key
;      If detected goto state B1
;
; A5.  If error status, flash LED (fast)
;
; A6.  Goto state 3
;
; Enter 'B' when UNTUNE is selected from the main idle loop
; In this case the operator wants to release the tuner so that
; it is operating straigth through. This is done by asserting
; START for 50 ms then releasing it. Since the tuner is off-line
; the LED is turned off. When done, return to the main idle loop.
;
; B1.  Untune key - disable tuner:
;      Turn LED off
;      Assert START
;      Delay 50 ms
;      Deassert START
;      Goto state A3
;    
; Enter 'C' to start the tune operation. START is asserted to the tuner
; for 500 ms. During this time, the tuner should assert KEY. This is an
; indication to the radio to supply RF. In response to KEY, we assert TX.
; If KEY is not asserted after 500 ms, we declare an error.
; While in the loop, flash the LED and check for UNTUNE. UNTUNE aborts the
; tune operation.
;
; C1.  Tune key:
;      Setup main timer for 500 ms delay
;
; C2.  Setup hardware timer for 100 ms delay
;
; C3.  Check for untune key
;      If detected, goto state E1
;
; C4.  If Key line asserted
;      Assert transmitter Tx line
;
; C5.  If hardware timer has not expired goto state C3
;
; C6.  Decrement main timer
;      If zero, goto state C8
;
; C7.  Toggle LED (slow flash)
;      Goto state C2 (reset hardware timer)
;
; C8.  Tuner START assert has expired (500 ms) and no error detected:
;      Clear START
; 
; C9.  If Key from tuner not detected, goto state E2 (error)
;
; Enter 'D' when tuning has started. In this state we wait for tuning to
; be completed up to 5 seconds. If KEY is not removed within 5 seconds, we
; declare an error (this time may need to be increased).
; When KEY goes high, we exit this state. While in this state we flash the
; LED and look for UNTUNE (which aborts the tune operation).
;
; D1.  Delay for AH-4 tuning operation
;      Set main timer to 5 seconds
;
; D2.  Set hardware timer to 100 ms
;
; D3.  If KEY from AH-4 no longer asserted, goto state F1
;      If UNTUNE asserted, goto state E1 (error)
;
; D4.  If hardware timer not zero, goto state D3
;
; D5.  Decrement main timer
;      If zero, goto state E2 (error)
;
; D6.  Toggle LED (slow flash)
;      Goto state D2
;
; Enter 'F' when tuning has been completed, as signified by deassertion of KEY
; When KEY is deasserted, deassert TX. If KEY is asserted again in less than
; 100 ms, this indicates that the tuner did not tune. In this case we abort the
; tune operation. If KEY is not reasserted, then the tune was successful and we
; can return to the main idle loop with the LED turned on.
;
; F1.  Deassert TX
;
; F2.  Set hardware timer to 100 ms
;
; F3.  If KEY is asserted, goto state E2 (error)
;
; F4.  If no timeout, goto state F3
;
; F5.  Turn LED on
;      Clear error status (just in case)
;
; F6.  Goto state A3
;
; Enter 'E1' for a user abort. Turn everything off and set error status so the
; LED flashes when in the main idle loop (state A).
;
; E1.  Error state, tune abort
;      Set error status
;      Deassert TX
;      Deassert STATUS
;      Goto state A3
;
; Enter 'E2' when an error is detected. Turn everything off and return to the
; main idle loop (state A).
;
; E2.  Error state, tune fail
;      Set LED off
;      Clear error status
;      Deassert TX
;      Deassert START
;      Goto state A3
;      
;**********************************************************************
;
		SUBTITLE		"Declarations"
		page
;	list      p=16c54B		; list directive to define processor
	#include <p16c5x.inc>		; processor specific variable definitions

	__CONFIG   _CP_OFF & _WDT_OFF & _XT_OSC

; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file. 
; See respective data sheet for additional information on configuration word.


;**********************************************************************
;***** CONSTANTS
;*** Production --- comment out for debug
;#define DEBUG 			; Comment out for production
		ifdef	DEBUG
;*** Debug      --- comment out for production
; Debug values allow simulation to proceed faster (albeit with much shorter times)
DELAY_100ms	EQU	0xf8	; Debug
DELAY_50ms	EQU	0xf8	; Debug
DELAY_1_5ms	EQU	0xf8	; Debug
DELAY_500ms	EQU	1	; Debug
DELAY_5sec	EQU	2	; Debug
PRESCALER	EQU	0x02	; Debug Divide by 4 (note - cannot make smaller otherwise simulation will fail)
AUDIO_DELAY	EQU	2	; Debug
		endif

		ifndef	DEBUG
DELAY_100ms	EQU	0x10	; Production - input to TMR0
DELAY_50ms	EQU	0x88	; Production - input to TMR0
DELAY_1_5ms	EQU	0xfb	; Production - input to TMR0
DELAY_500ms	EQU	5	; Production - input to TIMER_REG
DELAY_5sec	EQU	0x30	; Production - input to TIMER_REG
PRESCALER	EQU     0x07	; Divide by 256
AUDIO_DELAY	EQU	13
		endif


;**********************************************************************

;**********************************************************************
;***** VARIABLE DEFINITIONS
;  RA0 = Clock to EEPROM (not implemented)
;  RA1 = Not used
;  RA2 = ALC control output
;  RA3 = Tune audio output
;  RB0 = Data to EEPROM (not implemented)
;  RB1 = START output (Signal to AH4 to start tuning) set to assert
;  RB2 = KEY input (input from AH4 to cause Tx to engage) set if key asserted
;  RB3 = TX Key output (to radio to cause Tx to engage) set to transmit
;  RB4 = PSK31 Tx Key (input from MAX232 DTS)
;  RB5 = LED output (indicates tune status) clear to turn LED on
;  RB6 = Tune input (command from operator to start tuning) clear if asserted
;  RB7 = Untune input (command from operator to release tuning) clear if asserted
; RB:
UNTUNE		EQU	7	; 0=UNTUNE button pressed (in)
TUNE		EQU	6	; 0=TUNE button pressed (in)
LED		EQU	5	; 0=LED on (out)
LED_MASK	EQU	0x20	; Mask for LED (Bit location 5)
DTS		EQU	4	;
TX		EQU	3	; 1=Tx on (out)
AUDIO_MASK	EQU	0x08	; Mask for audio output (bit location 3)
AUDIO_MASK_BP	EQU	0x03	; Bit position for AUDIO_MASK
KEY		EQU	2	; 1=Key asserted (in)
ALC		EQU	2
START		EQU	1	; 1=Start asserted (out)
SI		EQU	0
;**********************************************************************

;**********************************************************************
; Storage locations
TIMER_REG	EQU	0x10		; For timer comparisons
TUNE_STATUS	EQU	0x11		; Status of tune operation
TUNE_STATUS_ERR	EQU	0x00		; 1 = abort (bit position)
DISABLE_PSK	EQU	0x01		; 2
AUDIO_COUNT	EQU	0x12
;**********************************************************************

		SUBTITLE		"State A"
		page
;**********************************************************************
		ORG     0x1FF		; processor reset vector
		goto    start
		ORG     0x000

; Start of program execution
; Initialize the hardware
;
; *******
; State A
; *******

start		
; Setup status register
; PA1:PA0 = 0 program page
		CLRW
		MOVWF	STATUS

; Set Timer prescaler to divide by 256 and assign it to the timer
; Basic clock rate to timer is (f0/4)/256
; For 2.4576 MHz, timer rate is 2400 Hz or 0.000417 seconds
; Max delay is 105ms (TMR0=256)

		MOVLW	PRESCALER	; Set prescaler
		OPTION			; Write to option register

; Set the following pins for output (all other pins are inputs)
; RB1, RB3, RB4, RB5
		MOVLW	B'11110010'	; All inputs except 0, 2, 3 output
		TRIS	PORTA
		MOVLW	B'11010101'	; 5, 3, 1 outputs
		TRIS	PORTB

; Initialize the output pins
; Turn LED on
		BCF	PORTB, LED
; Turn Tx off
		BCF	PORTB, TX
; Turn ALC off
		BCF	PORTA, ALC
; Turn Start off
		BCF	PORTB, START
; Clear tune status
		CLRF	TUNE_STATUS
; Reset the tone timer
		CALL	TONE_RESET

; Delay for 100 ms
; Start timer
		MOVLW	DELAY_100ms		; Load timer with 128 (50 ms delay)
		MOVWF	TMR0

; Wait for 100 ms
startup_wait
		MOVF	TMR0,W			; Check timer value
		BTFSS	STATUS, Z		; Is it zero?
		GOTO	startup_wait		; Not done, loop

		BSF	PORTB, LED		; Turn LED off

; Delay for 100 ms
; Start timer
		MOVLW	DELAY_100ms		; Load timer with 128 (50 ms delay)
		MOVWF	TMR0

; Wait for 100 ms
startup_wait2
		MOVF	TMR0,W			; Check timer value
		BTFSS	STATUS, Z		; Is it zero?
		GOTO	startup_wait2		; Not done, loop

		BCF	PORTB, LED		; Turn LED on
; Delay for 100 ms
; Start timer
		MOVLW	DELAY_100ms		; Load timer with 128 (50 ms delay)
		MOVWF	TMR0

; Wait for 100 ms
startup_wait3
		MOVF	TMR0,W			; Check timer value
		BTFSS	STATUS, Z		; Is it zero?
		GOTO	startup_wait3		; Not done, loop

		BSF	PORTB, LED		; Turn LED off


;**********************************************************************
; Check for test mode
; If the tune button is pressed, enter test mode
		BTFSS	PORTB, TUNE
		GOTO	TEST_MODE

;**********************************************************************
; Check for asserted PSK TX input
; If present, disable PSK TX
		BCF	TUNE_STATUS, DISABLE_PSK
		BTFSS	PORTB, DTS			; Disable if input is low (no MAX232 chip)
		BSF	TUNE_STATUS, DISABLE_PSK	; Disable

;**********************************************************************
; This is the start of the main loop
; We always come back here when an operation has been completed		
main_loop

; Flash LED quickly if error condition
		BCF	PORTA, ALC		; Turn ALC off
		BCF	PORTA, AUDIO_MASK_BP	; Set audio level low
		BTFSS	TUNE_STATUS, TUNE_STATUS_ERR
		goto	main_loop_1		; No error, don't flash LED

		BTFSS	TMR0, 7			; Skip if not time to flash
		GOTO	main_loop_1

		MOVF	PORTB, W		; Get port
		XORLW	LED_MASK		; Exclusive or LED bit
		MOVWF	PORTB			; Write it back
		CLRF	TMR0			; Reset timer
			
; Check for tune button press
main_loop_1
		BTFSS	PORTB, TUNE
		GOTO	tune_press

; Check for untune button press
; Disable DTS detection if UNTUNE was pressed during power-up.
; Note DTS may have already been disabled if it was asserted during power up.
		BTFSS	PORTB, UNTUNE
		BSF	TUNE_STATUS, DISABLE_PSK

; Check for Tx Key request from PSK31 (MAX 232 DTS input)

		BTFSC	PORTB, DTS		; If high (input low), don't transmit
		GOTO	dts_tx_off

; Don't transmit if the feature has been disabled
		BTFSC	TUNE_STATUS, DISABLE_PSK
		GOTO	dts_tx_off

		BSF	PORTB, TX		; Turn transmitter on
		GOTO	dts_tx

dts_tx_off	BCF	PORTB, TX		; Turn transmitter off
dts_tx

; Check for untune, if not loop
		BTFSC	PORTB, UNTUNE		; BTFSS doesn't work with bit 7
		GOTO	main_loop
; Fall through to untune

		SUBTITLE			"State B"
		page
; *******
; State B
; *******

;**********************************************************************
; Process Untune
process_untune
		BTFSS	PORTB, UNTUNE		; Wait for button release
		GOTO	process_untune
; Turn LED off
		BSF	PORTB, LED
; Turn transmitter off
		BCF	PORTB, TX
; Clear error status
		BCF	TUNE_STATUS, TUNE_STATUS_ERR
; Assert START
		BSF	PORTB, START
; Start timer
		MOVLW	DELAY_50ms		; Load timer with 128 (50 ms delay)
		MOVWF	TMR0
; Wait for 50 ms
untune_wait
		MOVF	TMR0,W			; Check timer value
		BTFSS	STATUS, Z		; Is it zero?
		GOTO	untune_wait		; Not done, loop
; Deassert START
		BCF	PORTB, START
; Start timer
		MOVLW	DELAY_100ms		; Load timer with 128 (50 ms delay)
		MOVWF	TMR0
; Wait for 100 ms
untune_wait_again
		MOVF	TMR0,W			; Check timer value
		BTFSS	STATUS, Z		; Is it zero?
		GOTO	untune_wait_again	; Not done, loop; Return to main loop
		GOTO	main_loop

		SUBTITLE			"State C"
		page
; *******
; State C
; *******
;**********************************************************************
; Process a tune operation
tune_press
		BTFSS	PORTB, TUNE		; Wait until button released
		GOTO	tune_press
		BCF	PORTB, TX		; Turn transmitter off
		MOVLW	DELAY_500ms		; Set timer for 5 intervals (500 ms)
		MOVWF	TIMER_REG
		BSF	PORTB, START		; Assert start
		BSF	PORTA, ALC		; Turn on ALC voltage

; If key is detected, enable the transmitter
; Key line goes high when the AH-4 grounds the keying line
tune_start_loop
		MOVLW	DELAY_100ms		; Set 100 ms delay
		MOVWF	TMR0
tune_loop
		CALL	TONE_GENERATION		; Generate tone
; Check for untune key and abort if detected
		BTFSS	PORTB, UNTUNE
		GOTO	tune_abort

; No error, enagage transmitter if tuner has asserted KEY
		BTFSS	PORTB, KEY		; Don't Tx if line is low
		GOTO	skip_tx			; Key line is low
		BSF	PORTB, TX		; Key line is high, enable Tx
skip_tx
; Now check for timeout, exit loop after 500 ms
		MOVF	TMR0,W			; Check timer
		BTFSS	STATUS, Z		; Update main counter if zero
		GOTO	tune_loop		; Loop if no timer update
		DECF	TIMER_REG, F		; Decrement the timer register
		BTFSC	STATUS, Z		; Done if zero
		GOTO	exit_tune_start
; Toggle LED
		MOVF	PORTB, W		; Get port
		XORLW	LED_MASK		; Exclusive or LED bit
		MOVWF	PORTB			; Write it back
		GOTO	tune_start_loop		; Continue with loop


;**********************************************************************
exit_tune_start
; Start has been asserted to the tuner, okay to remove it
		BCF	PORTB, START
; Toggle LED since we'll toggle it again at the end of this loop
		MOVF	PORTB, W		; Get port
		XORLW	LED_MASK		; Exclusive or LED bit
		MOVWF	PORTB			; Write it back

; If key has not been asserted, then an error has occurred
		BTFSS	PORTB, KEY
		GOTO	tune_abort

		SUBTITLE			"State D"
		page
; *******
; State D
; *******

; The AH-4 is now tuning
; Wait for the AH-4 to finish, continue with slow LED flash
		MOVLW	DELAY_5sec		; Wait for about 5 seconds max
		MOVWF	TIMER_REG

;**********************************************************************
exit_tune_loop
		MOVLW	DELAY_100ms		; Set 100 ms delay
		MOVWF	TMR0

; Check for untune key and abort if detected
exit_loop
		CALL	TONE_GENERATION		; Generate tone
		BTFSS	PORTB, UNTUNE
		GOTO	tune_abort

; Now wait for KEY from the AH-4 to be removed
		BTFSS	PORTB, KEY		; Wait for key to be removed
		GOTO	key_off

; Now check for timeout
		MOVF	TMR0, W			; Check timer
		BTFSS	STATUS, Z		; Update main counter if zero
		GOTO	exit_loop		; Loop if no timer update
		DECF	TIMER_REG, F		; Decrement the timer register
		BTFSC	STATUS, Z		; Error if zero
		GOTO	tune_abort

; Toggle LED
		MOVF	PORTB, W		; Get port
		XORLW	LED_MASK		; Exclusive or LED bit
		MOVWF	PORTB			; Write it back
		GOTO	exit_tune_loop		; Continue with loop

		SUBTITLE			"State F"
		page
; *******
; State F
; *******

;**********************************************************************
; A rising edge has been detected on the key line from the tuner
; This could mean that the tune operation has been completed or that
; the tuner could not tune. If the pulse is less than 100 ms, then
; the tune was not successful.
key_off
		BCF	PORTB, TX		; Remove Tx
		
; Delay for 1.5 ms (or so) to allow the signal line to settle
		MOVLW	DELAY_1_5ms
		MOVWF	TMR0
key_off_delay_loop		
		MOVF	TMR0, W			; Check timer
		BTFSS	STATUS, Z
		GOTO	key_off_delay_loop

; Setup timer for 100 ms
		MOVLW	DELAY_100ms		; Set 100 ms delay
		MOVWF	TMR0

; If key goes low before timeout, abort
key_off_loop
		BTFSC	PORTB, KEY
		GOTO	tune_fail

; Now time for 100 ms
		MOVF	TMR0, W	
		BTFSS	STATUS, Z		; Loop if not zero
		goto	key_off_loop

;**********************************************************************
; Timeout - tune was successful
		BCF	PORTB, LED		; Turn LED on
		BCF	TUNE_STATUS, TUNE_STATUS_ERR		; Reset status
		GOTO	main_loop		; Done		

		SUBTITLE			"Error States"
		page
; ********
; State E1
; ********

;**********************************************************************
; Go here if the tune operation is aborted
tune_abort
		BSF	TUNE_STATUS, TUNE_STATUS_ERR
		BCF	PORTB, TX		; Make sure Tx is off!
		BCF	PORTB, START		; Clear Start
		GOTO	main_loop

; ********
; State E2
; ********

;**********************************************************************
; Go here if the tune operation failed
tune_fail
		BCF	TUNE_STATUS, TUNE_STATUS_ERR
		BCF	PORTB, TX		; Make sure Tx is off!
		BCF	PORTB, START		; Clear Start
		BSF	PORTB, LED		; Force LED off
		GOTO	main_loop

		SUBTITLE		"Test Mode"
		page
TEST_MODE
; If tune is pressed, assert TX and ALC else clear TX
		BTFSS	PORTB, TUNE
		BSF	PORTB, TX
		BTFSC	PORTB, TUNE
		BCF	PORTB, TX

		BTFSS	PORTB, TUNE
		BSF	PORTA, ALC		; Turn ALC on
		BTFSC	PORTB, TUNE
		BCF	PORTA, ALC		; Turn ALC off

; If untune is pressed, assert START, else clear START
		BTFSS	PORTB, UNTUNE
		BSF	PORTB, START
		BTFSC	PORTB, UNTUNE
		BCF	PORTB, START

; If KEY is asserted, light LED otherwise extinguish it
		BTFSS	PORTB, KEY
		BCF	PORTB, LED
		BTFSC	PORTB, KEY
		BSF	PORTB, LED
		GOTO	TEST_MODE	; Loop
		
; Generate a tone
TONE_GENERATION
; First determine if we should toggle the audio output line
		DECF	AUDIO_COUNT, 1		; Decrement counter
		MOVF	AUDIO_COUNT, W		; Test counter
		BTFSS	STATUS, Z		; Zero?
		GOTO	skip_tone		; No, don't toggle
		
		MOVF	PORTA, W		; Get port
		XORLW	AUDIO_MASK		; Exclusive or audio bit
		MOVWF	PORTA			; Write it back
; Reset the counter
TONE_RESET
		MOVLW	AUDIO_DELAY
		MOVWF	AUDIO_COUNT
skip_tone
		RETLW	0			; Return
		
		SUBTITLE		"Assembler Information"
		END



